/*
 * blueMail offline mail reader
 * packet statistics

 Copyright (c) 2000 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */


#include "bmail.h"


packet_stat::packet_stat (bmail *bm)
{
  FILE *statfile;
  pktstat stat;

  ro = bm->resourceObject;
  anchor = last = NULL;

  if ((statfile = fopen(ro->get(statisticsFile), "rb")))
  {
    while (fread(&stat, sizeof(pktstat), 1, statfile)) newStat(&stat);
    fclose(statfile);
  }
}


packet_stat::~packet_stat ()
{
  FILE *statfile;
  pktstat *next, *i = anchor;

  if (anchor && (statfile = fopen(ro->get(statisticsFile), "wb")))
  {
    while (i)
    {
      next = i->next;

      if (i->used)
      {
        i->used = false;
        fwrite(i, sizeof(pktstat), 1, statfile);
      }

      delete i;
      i = next;
    }
    fclose(statfile);
  }
}


void packet_stat::newStat (pktstat *stat)
{
  pktstat *nstat = new pktstat;

  if (!anchor) anchor = nstat;
  stat->next = NULL;
  *nstat = *stat;
  if (last) last->next = nstat;
  last = nstat;
}


void packet_stat::addStat (time_t date, int total, int unread)
{
  pktstat *i = anchor;

  while (i)
  {
    if (i->pktDate == date)
    {
      i->msgsTotal = total;
      i->msgsUnread = unread;
      i->used = true;
      return;
    }
    i = i->next;
  }

  pktstat stat;
  stat.pktDate = date;
  stat.msgsTotal = total;
  stat.msgsUnread = unread;
  stat.used = true;
  newStat(&stat);
}


bool packet_stat::getStat (time_t date, int *total, int *unread) const
{
  pktstat *i = anchor;

  while (i)
  {
    if (i->pktDate == date)
    {
      *total = i->msgsTotal;
      *unread = i->msgsUnread;
      i->used = true;
      return true;
    }
    i = i->next;
  }

  return false;
}
